#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from unittest import TestCase
from datetime import date, datetime
from json import loads

from agspiel.api.historic_ag import HistorischeAg

class TestHistorischeAg(TestCase):
    def setUp(self):
        f = open("testhistoricag.json", "r")
        api_data = loads(f.read())
        f.close()
        self.ag = HistorischeAg(wkn=175353, datum=date(2021, 7, 23), api_data=api_data)

    def test_wkn(self):
        self.assertEqual(self.ag.wkn, 175353)
        self.assertIsInstance(self.ag.wkn, int)

    def test_datum(self):
        self.assertEqual(self.ag.datum, date(2021, 7, 23))
        self.assertIsInstance(self.ag.datum, date)

    def test_name(self):
        self.assertEqual(self.ag.name, "King Kompany")
        self.assertIsInstance(self.ag.name, str)

    # def test_logo(self):
    #     self.fail()

    def test_gruendung(self):
        self.assertEqual(self.ag.gruendung, datetime(year=2020, month=2, day=4))
        self.assertIsInstance(self.ag.gruendung, datetime)

    def test_aktienanzahl(self):
        self.assertEqual(self.ag.aktienanzahl, 4000000)
        self.assertIsInstance(self.ag.aktienanzahl, int)

#   def test_in_liquidation(self):
#       self.fail()

    def test_kurs(self):
        self.assertEqual(self.ag.kurs, 630)
        self.assertIsInstance(self.ag.kurs, float)

    def test_brief(self):
        self.assertEqual(self.ag.brief, 630)
        self.assertIsInstance(self.ag.brief, float)

    def test_geld(self):
        self.assertEqual(self.ag.geld, 601.99)
        self.assertIsInstance(self.ag.geld, float)

    def test_brief_stueckzahl(self):
        self.assertEqual(self.ag.brief_stueckzahl, 16287)
        self.assertIsInstance(self.ag.brief_stueckzahl, int)

    def test_geld_stueckzahl(self):
        self.assertEqual(self.ag.geld_stueckzahl, 10945)
        self.assertIsInstance(self.ag.geld_stueckzahl, int)

    def test_sw_aktie(self):
        self.assertEqual(self.ag.sw_aktie, 352.26)
        self.assertIsInstance(self.ag.sw_aktie, float)

    def test_bbw_aktie(self):
        self.assertEqual(self.ag.bbw_aktie, 482.78)
        self.assertIsInstance(self.ag.bbw_aktie, float)

    def test_fp_aktie(self):
        self.assertEqual(self.ag.fp_aktie, 436.23)
        self.assertIsInstance(self.ag.fp_aktie, float)

    def test_bw_aktie(self):
        self.assertEqual(self.ag.bw_aktie, 587.09)
        self.assertIsInstance(self.ag.bw_aktie, float)

#   def test_kgv(self):
#       self.fail()

    def test_spread(self):
        self.assertEqual(self.ag.spread, 0.04446031746031742)
        self.assertIsInstance(self.ag.spread, float)

    def test_alter(self):
        self.assertEqual(self.ag.alter, (datetime.now() - self.ag.gruendung).days)
        self.assertIsInstance(self.ag.alter, int)

#   def test_tagesvolumen(self):
#       self.fail()

    def test_boersenwert(self):
        self.assertEqual(self.ag.boersenwert, 2520000000)
        self.assertIsInstance(self.ag.boersenwert, float)

    def test_buchwert(self):
        self.assertEqual(self.ag.buchwert, 2348360222.54)
        self.assertIsInstance(self.ag.buchwert, float)

    def test_depotwert(self):
        self.assertEqual(self.ag.depotwert, 1856199270.98)
        self.assertIsInstance(self.ag.depotwert, float)

    def test_bargeld(self):
        self.assertEqual(self.ag.bargeld, 492160951.56)
        self.assertIsInstance(self.ag.bargeld, float)

    def test_highscore(self):
        self.assertEqual(self.ag.highscore, 49)
        self.assertIsInstance(self.ag.highscore, int)

    def test_highscore_groesse(self):
        self.assertEqual(self.ag.highscore_groesse, 178)
        self.assertIsInstance(self.ag.highscore_groesse, int)

    def test_highscore_wachstum(self):
        self.assertEqual(self.ag.highscore_wachstum, 174)
        self.assertIsInstance(self.ag.highscore_wachstum, int)

#   def test_highscore_newcomer(self):
#       self.fail()

    def test_agsx_punkte(self):
        self.assertEqual(self.ag.agsx_punkte, 1302)
        self.assertIsInstance(self.ag.agsx_punkte, int)

    def test_in_agsx(self):
        self.assertEqual(self.ag.in_agsx, False)
        self.assertIsInstance(self.ag.in_agsx, bool)

    def test_handelsaktivitaet(self):
        self.assertEqual(self.ag.handelsaktivitaet, 94)
        self.assertIsInstance(self.ag.handelsaktivitaet, int)

#   def test_ceo(self):
#       self.fail()
#
#   def test_aktien(self):
#       self.fail()
#
#   def test_anleihen(self):
#       self.fail()
#
#   def test_kredite(self):
#       self.fail()
#
#   def test_zertifikate(self):
#       self.fail()
#
#   def test_orders(self):
#       self.fail()
#
#   def test_aktionaere(self):
#       self.fail()
#
#   def test_kes(self):
#       self.fail()
#
#   def test_khs(self):
#       self.fail()
#
#   def test_dividende(self):
#       self.fail()
#
#   def test_max_zertis(self):
#       self.fail()
#
#   def test_uebernahmeschutz(self):
#       self.fail()
#
#   def test_tages_hoch(self):
#       self.fail()
#
#   def test_tages_tief(self):
#       self.fail()
#
#   def test_kurs_14d(self):
#       self.fail()
#
#   def test_kurs_30d(self):
#       self.fail()
#
#   def test_kurs_60d(self):
#       self.fail()
#
#   def test_kurs_90d(self):
#       self.fail()
#
#   def test_bw_14d(self):
#       self.fail()
#
#   def test_bw_30d(self):
#       self.fail()
#
#   def test_bw_60d(self):
#       self.fail()
#
#   def test_bw_90d(self):
#       self.fail()
#
#   def test_fp_14d(self):
#       self.fail()
#
#   def test_fp_30d(self):
#       self.fail()
#
#   def test_fp_60d(self):
#       self.fail()
#
#   def test_fp_90d(self):
#       self.fail()


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

import json
from unittest import TestCase
from datetime import datetime, date
from agspiel.api.ag import Ag
from agspiel.api.data import Data
from agspiel.api.ceo import Ceo
from agspiel.api.aktie import Aktie, Aktionaer
from agspiel.api.kapitalmassnahme import Kapitalerhoehung, Kapitalherabsetzung
from agspiel.api.anleihe import Anleihe, Kredit
from agspiel.api.zertifikat import Zertifikat
from agspiel.api.order import Order
from agspiel.api.index import Index
from bs4 import BeautifulSoup

class TestAg(TestCase):
    def setUp(self):
        f = open("testag.txt", "rb")
        web_data = Data(data=BeautifulSoup(f.read(), "html.parser"), update=lambda: self.data)
        f.close()
        f = open("testapi.txt", "r")
        api_data = Data(data=json.loads(f.read()), update=lambda: self.data)
        f.close()
        f = open("testchronik.txt", "rb")
        chronik_data = Data(data=BeautifulSoup(f.read(), "html.parser"), update=lambda: self.data) # FIXME Chronik Data not from 175353
        f.close()
        self.ag = Ag(wkn=175353, api_data=api_data, web_data=web_data, chronik_data=chronik_data)

    def test_wkn(self):
        self.assertEqual(self.ag.wkn, 175353)
        self.assertIsInstance(self.ag.wkn, int)

    def test_name(self):
        self.assertEqual(self.ag.name, "King Kompany")
        self.assertIsInstance(self.ag.name, str)

    def test_logo(self):
        self.assertEqual(self.ag.logo, "https://www.ag-spiel.de/uploads/83275.jpg?t=1594897")
        self.assertIsInstance(self.ag.name, str)

    def test_gruendung(self):
        self.assertEqual(self.ag.gruendung, datetime(year=2020, month=2, day=4, hour=16, minute=25, second=5))
        self.assertIsInstance(self.ag.gruendung, datetime)

    def test_aktienanzahl(self):
        self.assertEqual(self.ag.aktienanzahl, 2000000)
        self.assertIsInstance(self.ag.aktienanzahl, int)

    def test_in_liquidation(self):
        self.assertEqual(self.ag.in_liquidation, False)
        self.assertIsInstance(self.ag.in_liquidation, bool)

    def test_in_kapitalerhoehung(self):
        self.assertEqual(self.ag.in_kapitalerhoehung, True)
        self.assertIsInstance(self.ag.in_kapitalerhoehung, bool)

    def test_kurs(self):
        self.assertEqual(self.ag.kurs, 262)
        self.assertIsInstance(self.ag.kurs, float)

    def test_brief(self):
        self.assertEqual(self.ag.brief, 299)
        self.assertIsInstance(self.ag.brief, float)

    def test_geld(self):
        self.assertEqual(self.ag.geld, 0)
        self.assertIsInstance(self.ag.geld, float)

    def test_brief_stueckzahl(self):
        self.assertEqual(self.ag.brief_stueckzahl, 112291)
        self.assertIsInstance(self.ag.brief_stueckzahl, int)

    def test_geld_stueckzahl(self):
        self.assertEqual(self.ag.geld_stueckzahl, 0)
        self.assertIsInstance(self.ag.geld_stueckzahl, int)

    def test_sw_aktie(self):
        self.assertEqual(self.ag.sw_aktie, 52.30)
        self.assertIsInstance(self.ag.sw_aktie, float)

    def test_bbw_aktie(self):
        self.assertEqual(self.ag.bbw_aktie, 59.01)
        self.assertIsInstance(self.ag.bbw_aktie, float)

    def test_fp_aktie(self):
        self.assertEqual(self.ag.fp_aktie,  4038629.29)
        self.assertIsInstance(self.ag.fp_aktie, float)

    def test_bw_aktie(self):
        self.assertEqual(self.ag.bw_aktie, 114.47)
        self.assertIsInstance(self.ag.bw_aktie, float)

    def test_kgv(self):
        self.assertEqual(self.ag.kgv, 115.59)
        self.assertIsInstance(self.ag.kgv, float)

    def test_spread(self):
        self.assertEqual(self.ag.spread, 0)
        self.assertIsInstance(self.ag.spread, int)

    def test_alter(self):
        self.assertEqual(self.ag.alter, (datetime.now() - self.ag.gruendung).days)
        self.assertIsInstance(self.ag.alter, int)

    def test_tagesvolumen(self):
        self.assertEqual(self.ag.tagesvolumen, 0)
        self.assertIsInstance(self.ag.tagesvolumen, float)

    def test_boersenwert(self):
        self.assertEqual(self.ag.boersenwert, 524000000)
        self.assertIsInstance(self.ag.boersenwert, float)

    def test_buchwert(self):
        self.assertEqual(self.ag.buchwert, 228945121.15)
        self.assertIsInstance(self.ag.buchwert, float)

    def test_depotwert(self):
        self.assertEqual(self.ag.depotwert, 179176731.12)
        self.assertIsInstance(self.ag.depotwert, float)

    def test_bargeld(self):
        self.assertEqual(self.ag.bargeld, 49768390.03)
        self.assertIsInstance(self.ag.bargeld, float)

    def test_highscore(self):
        self.assertEqual(self.ag.highscore, 245)
        self.assertIsInstance(self.ag.highscore, int)

    def test_highscore_groesse(self):
        self.assertEqual(self.ag.highscore_groesse, 320)
        self.assertIsInstance(self.ag.highscore_groesse, int)

    def test_highscore_wachstum(self):
        self.assertEqual(self.ag.highscore_wachstum, 282)
        self.assertIsInstance(self.ag.highscore_wachstum, int)

    def test_highscore_newcomer(self):
        self.assertEqual(self.ag.highscore_newcomer, 0)
        self.assertIsInstance(self.ag.highscore_newcomer, int)

    def test_agsx_punkte(self):
        self.assertEqual(self.ag.agsx_punkte, 1200)
        self.assertIsInstance(self.ag.agsx_punkte, int)

    def test_in_agsx(self):
        self.assertEqual(self.ag.in_agsx, True)
        self.assertIsInstance(self.ag.in_agsx, bool)

    def test_handelsaktivitaet(self):
        self.assertEqual(self.ag.handelsaktivitaet, 43)
        self.assertIsInstance(self.ag.handelsaktivitaet, int)

    def test_ceo(self):
        self.assertIsInstance(self.ag.ceo, Ceo)
        self.assertEqual(self.ag.ceo.name, "KingKevin23")
        self.assertIsInstance(self.ag.ceo.name, str)

        # Tests zum Index
        self.assertIsInstance(self.ag.ceo.index, Index)
        self.assertEqual(self.ag.ceo.index.id, 2349)
        self.assertIsInstance(self.ag.ceo.index.id, int)
        self.assertEqual(self.ag.ceo.index.name, "Freibeuter")
        self.assertIsInstance(self.ag.ceo.index.name, str)
        self.assertEqual(self.ag.ceo.index.highscore, 8)
        self.assertIsInstance(self.ag.ceo.index.highscore, int)
        self.assertEqual(self.ag.ceo.index.punkte, 381)
        self.assertIsInstance(self.ag.ceo.index.punkte, int)
        self.assertEqual(self.ag.ceo.index.gruendung_datum, datetime(year=2019, month=11, day=19, hour=0, minute=10, second=12))
        self.assertIsInstance(self.ag.ceo.index.gruendung_datum, datetime)

        self.assertEqual(self.ag.ceo.registrierung_datum, datetime(year=2020, month=2, day=4, hour=16, minute=24))
        self.assertIsInstance(self.ag.ceo.registrierung_datum, datetime)
        self.assertEqual(self.ag.ceo.gesperrt, False)
        self.assertIsInstance(self.ag.ceo.gesperrt, bool)
        self.assertEqual(self.ag.ceo.userprojekt, True)
        self.assertIsInstance(self.ag.ceo.userprojekt, bool)

    def test_aktien(self):
        for i in self.ag.aktien:
            self.assertIsInstance(i, Aktie)
        self.assertEqual(self.ag.aktien[0].wkn, 146244)
        self.assertIsInstance(self.ag.aktien[0].wkn, int)
        self.assertEqual(self.ag.aktien[0].stueckzahl, 5)
        self.assertIsInstance(self.ag.aktien[0].stueckzahl, int)
        self.assertEqual(self.ag.aktien[1].wkn, 175541)
        self.assertIsInstance(self.ag.aktien[1].wkn, int)
        self.assertEqual(self.ag.aktien[1].stueckzahl, 12615)
        self.assertIsInstance(self.ag.aktien[1].stueckzahl, int)

    def test_anleihen(self):
        for i in self.ag.anleihen:
            self.assertIsInstance(i, Anleihe)
        self.assertEqual(self.ag.anleihen[0].betrag, 10000000)
        self.assertIsInstance(self.ag.anleihen[0].betrag, int)
        self.assertEqual(self.ag.anleihen[0].zins, 0.36)
        self.assertIsInstance(self.ag.anleihen[0].zins, float)
        self.assertEqual(self.ag.anleihen[0].auszahlung_datum, datetime(year=2020, month=8, day=24, hour=15, minute=36, second=2))
        self.assertIsInstance(self.ag.anleihen[0].auszahlung_datum, datetime)
        self.assertEqual(self.ag.anleihen[0].laufzeit, 5)
        self.assertIsInstance(self.ag.anleihen[0].laufzeit, int)

    def test_kredite(self):
        for i in self.ag.kredite:
            self.assertIsInstance(i, Kredit)
        self.assertEqual(self.ag.kredite[0].betrag, 10000000)
        self.assertIsInstance(self.ag.kredite[0].betrag, int)
        self.assertEqual(self.ag.kredite[0].zins, 0.32)
        self.assertIsInstance(self.ag.kredite[0].zins, float)
        self.assertEqual(self.ag.kredite[0].rueckzahlung_datum, datetime(year=2020, month=8, day=14, hour=15, minute=1, second=33))
        self.assertIsInstance(self.ag.kredite[0].rueckzahlung_datum, datetime)
        self.assertEqual(self.ag.kredite[0].laufzeit, 5)
        self.assertIsInstance(self.ag.kredite[0].laufzeit, int)

    def test_zertifikate(self):
        for i in self.ag.zertifikate:
            self.assertIsInstance(i, Zertifikat)
        self.assertEqual(self.ag.zertifikate[0].betrag, 234.43)
        self.assertIsInstance(self.ag.zertifikate[0].betrag, float)
        self.assertEqual(self.ag.zertifikate[0].typ, "call")
        self.assertIsInstance(self.ag.zertifikate[0].typ, str)
        self.assertEqual(self.ag.zertifikate[0].hebel, 1.092)
        self.assertIsInstance(self.ag.zertifikate[0].hebel, float)
        self.assertEqual(self.ag.zertifikate[0].punkte, 23443)
        self.assertIsInstance(self.ag.zertifikate[0].punkte, int)
        self.assertEqual(self.ag.zertifikate[0].ablaufdatum, datetime(year=2020, month=8, day=10, hour=17, minute=48, second=1))
        self.assertIsInstance(self.ag.zertifikate[0].ablaufdatum, datetime)

    def test_orders(self):
        for i in self.ag.orders:
            self.assertIsInstance(i, Order)
        self.assertEqual(self.ag.orders[0].typ, "sell")
        self.assertIsInstance(self.ag.orders[0].typ, str)
        self.assertEqual(self.ag.orders[0].limit, 299)
        self.assertIsInstance(self.ag.orders[0].limit, float)
        self.assertEqual(self.ag.orders[0].stueckzahl, 112291)
        self.assertIsInstance(self.ag.orders[0].stueckzahl, int)
        self.assertEqual(self.ag.orders[0].orderregel, False)
        self.assertIsInstance(self.ag.orders[0].orderregel, bool)
        self.assertEqual(self.ag.orders[0].systembank, False)
        self.assertIsInstance(self.ag.orders[0].systembank, bool)
        self.assertEqual(self.ag.orders[0].datum, datetime(year=2020, month=8, day=23, hour=13, minute=3, second=25))
        self.assertIsInstance(self.ag.orders[0].datum, datetime)

    def test_aktionaere(self):
        temp = self.ag.aktionaere
        for i in temp:
            self.assertIsInstance(i, Aktionaer)
        self.assertEqual(len(temp), 53)
        self.assertEqual(temp[0].wkn, 103279)
        self.assertIsInstance(temp[0].wkn, int)
        self.assertEqual(temp[0].stueckzahl, 121495)
        self.assertIsInstance(temp[0].stueckzahl, int)

    def test_kes(self):
        kes = self.ag.kes
        for ke in kes:
            self.assertIsInstance(ke, Kapitalerhoehung)

        self.assertEqual(len(kes), 11)

        # lastest ke
        self.assertEqual(kes[0].datum, date(year=2020, month=5, day=12))
        self.assertIsInstance(kes[0].datum, date)
        self.assertEqual(kes[0].stueckzahl, 96000)
        self.assertIsInstance(kes[0].stueckzahl, int)
        self.assertEqual(kes[0].kurs, 180600.00)
        self.assertIsInstance(kes[0].kurs, float)
        self.assertEqual(kes[0].summe, 17337600000.00)
        self.assertIsInstance(kes[0].summe, float)

        # special ke with n/a values
        self.assertEqual(kes[2].datum, date(year=2018, month=6, day=27))
        self.assertIsInstance(kes[2].datum, date)
        self.assertEqual(kes[2].stueckzahl, 0)
        self.assertIsInstance(kes[2].stueckzahl, int)
        self.assertIsNone(kes[2].kurs)
        self.assertEqual(kes[2].summe, 0)
        self.assertIsInstance(kes[2].summe, float)

        # first ke
        last = len(kes) - 1
        self.assertEqual(kes[last].datum, date(year=2014, month=5, day=18))
        self.assertIsInstance(kes[last].datum, date)
        self.assertEqual(kes[last].stueckzahl, 550000)
        self.assertIsInstance(kes[last].stueckzahl, int)
        self.assertEqual(kes[last].kurs, 113.99)
        self.assertIsInstance(kes[last].kurs, float)
        self.assertEqual(kes[last].summe, 62694500.00)
        self.assertIsInstance(kes[last].summe, float)

    def test_khs(self):
        khs = self.ag.khs
        for kh in khs:
            self.assertIsInstance(kh, Kapitalherabsetzung)

        self.assertEqual(len(khs), 266)

        # lastest kh
        self.assertEqual(khs[0].datum, date(year=2021, month=4, day=11))
        self.assertIsInstance(khs[0].datum, date)
        self.assertEqual(khs[0].stueckzahl, 1473)
        self.assertIsInstance(khs[0].stueckzahl, int)
        self.assertEqual(khs[0].kurs, 173130.00)
        self.assertIsInstance(khs[0].kurs, float)
        self.assertEqual(khs[0].summe, 255020490.00)
        self.assertIsInstance(khs[0].summe, float)

        # special kh with n/a values
        self.assertEqual(khs[2].datum, date(year=2021, month=4, day=6))
        self.assertIsInstance(khs[2].datum, date)
        self.assertEqual(khs[2].stueckzahl, 0)
        self.assertIsInstance(khs[2].stueckzahl, int)
        self.assertIsNone(khs[2].kurs)
        self.assertEqual(khs[2].summe, 0)
        self.assertIsInstance(khs[2].summe, float)

        # first kh
        last = len(khs) - 1
        self.assertEqual(khs[last].datum, date(year=2014, month=6, day=7))
        self.assertIsInstance(khs[last].datum, date)
        self.assertEqual(khs[last].stueckzahl, 550000)
        self.assertIsInstance(khs[last].stueckzahl, int)
        self.assertEqual(khs[last].kurs, 116.07)
        self.assertIsInstance(khs[last].kurs, float)
        self.assertEqual(khs[last].summe, 63838500.00)
        self.assertIsInstance(khs[last].summe, float)

    def test_dividende(self):
        self.assertEqual(self.ag.dividende, 0)
        self.assertIsInstance(self.ag.dividende, float)

    def test_max_zertis(self):
        self.assertEqual(self.ag.max_zertis, 5)
        self.assertIsInstance(self.ag.max_zertis, int)

    def test_uebernahmeschutz(self):
        self.assertEqual(self.ag.uebernahmeschutz, False)
        self.assertIsInstance(self.ag.uebernahmeschutz, bool)

    def test_tages_hoch(self):
        self.assertEqual(self.ag.tages_hoch, 423.35)
        self.assertIsInstance(self.ag.tages_hoch, float)

    def test_tages_tief(self):
        self.assertEqual(self.ag.tages_tief, 338.68)
        self.assertIsInstance(self.ag.tages_tief, float)

    def test_kurs_14d(self):
        self.assertEqual(self.ag.kurs_14d, 38.24)
        self.assertIsInstance(self.ag.kurs_14d, float)

    def test_kurs_30d(self):
        self.assertEqual(self.ag.kurs_30d, -0.39)
        self.assertIsInstance(self.ag.kurs_30d, float)

    def test_kurs_60d(self):
        self.assertEqual(self.ag.kurs_60d, 47.26)
        self.assertIsInstance(self.ag.kurs_60d, float)

    def test_kurs_90d(self):
        self.assertEqual(self.ag.kurs_90d, 35.47)
        self.assertIsInstance(self.ag.kurs_90d, float)

    def test_bw_14d(self):
        self.assertEqual(self.ag.bw_14d, -12.32)
        self.assertIsInstance(self.ag.bw_14d, float)

    def test_bw_30d(self):
        self.assertEqual(self.ag.bw_30d, 4.74)
        self.assertIsInstance(self.ag.bw_30d, float)

    def test_bw_60d(self):
        self.assertIsNone(self.ag.bw_60d)
        self.assertIsInstance(self.ag.bw_60d, type(None))

    def test_bw_90d(self):
        self.assertEqual(self.ag.bw_90d, 68.96)
        self.assertIsInstance(self.ag.bw_90d, float)

    def test_fp_14d(self):
        self.assertEqual(self.ag.fp_14d, 3585.06)
        self.assertIsInstance(self.ag.fp_14d, float)

    def test_fp_30d(self):
        self.assertEqual(self.ag.fp_30d, 5.51)
        self.assertIsInstance(self.ag.fp_30d, float)

    def test_fp_60d(self):
        self.assertEqual(self.ag.fp_60d, 0.00)
        self.assertIsInstance(self.ag.fp_60d, float)

    def test_fp_90d(self):
        self.assertIsNone(self.ag.fp_90d)
        self.assertIsInstance(self.ag.fp_90d, type(None))

    def test_ordner(self):
        self.assertEqual(self.ag.ordner, "Allgemein")
        self.assertIsInstance(self.ag.ordner, str)

    def test_kauf_kurs(self):
        self.assertEqual(self.ag.kauf_kurs, None)
        self.assertIsInstance(self.ag.kauf_kurs, type(None))

    def test_kauf_anzahl(self):
        self.assertEqual(self.ag.kauf_anzahl, 0)
        self.assertIsInstance(self.ag.kauf_anzahl, int)


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime, date

class HistorischeAg:

    # Currently not included stats:
    # UserID, IndexID, Premiumstatus, Übernahmeschutz, AGSXDurchschnittsplatzierung, Wachstumsdurchschnittsplatz

    def __init__(self, wkn:int, datum:date, api_data:dict):
        self._wkn:int = wkn
        self._datum:date = datum
        self._api_data:dict = api_data

    @property
    def wkn(self) -> int:
        return self._wkn

    @property
    def datum(self) -> date:
        return self._datum

    @property
    def name(self) -> str:
        return self._api_data.get("AGName")

    # @property
    # def logo(self) -> str:
    #     pass

    @property
    def gruendung(self) -> datetime:
        return datetime.strptime(self._api_data.get("Gruendungsdatum"), "%Y-%m-%d")

    @property
    def aktienanzahl(self) -> int:
        return self._api_data.get("Aktienanzahl")

#   @property
#   def in_liquidation(self) -> bool:
#       pass

    @property
    def kurs(self) -> float:
        return self._api_data.get("Kurs_Wert")

    @property
    def brief(self) -> float:
        return self._api_data.get("Briefkurs_Wert")

    @property
    def geld(self) -> float:
        return self._api_data.get("Geldkurs_Wert")

    @property
    def brief_stueckzahl(self) -> int:
        return int(self._api_data.get("Briefvolumen"))

    @property
    def geld_stueckzahl(self) -> int:
        return int(self._api_data.get("Geldvolumen"))

    @property
    def sw_aktie(self) -> float:
        return self._api_data.get("SW_Wert")

    @property
    def bbw_aktie(self) -> float:
        return self._api_data.get("BBW_Wert")

    @property
    def fp_aktie(self) -> float:
        return self._api_data.get("FP_Wert")

    @property
    def bw_aktie(self) -> float:
        return self._api_data.get("BW_Wert")

#   @property
#   def kgv(self) -> float:
#       pass

    @property
    def spread(self) -> float:
        if self.geld == 0 or self.brief == 0:
            return 0
        else:
            return 1 - (self.geld / self.brief)

    @property
    def alter(self) -> int:
        return (datetime.now() - self.gruendung).days

#   @property
#   def tagesvolumen(self) -> float:
#       pass

    @property
    def boersenwert(self) -> float:
        return self.kurs * self.aktienanzahl

    @property
    def buchwert(self) -> float:
        return self.bargeld + self.depotwert

    @property
    def depotwert(self) -> float:
        return self._api_data.get("Aktienbuchwert_Wert") + self._api_data.get("Anleihen_Wert") + self._api_data.get("Zertifikate_Wert")

    @property
    def bargeld(self) -> float:
        return self._api_data.get("Cash_Wert")

    @property
    def highscore(self) -> int:
        return self._api_data.get("Platz")

    @property
    def highscore_groesse(self) -> int:
        return self._api_data.get("Groessenplatz")

    @property
    def highscore_wachstum(self) -> int:
        return self._api_data.get("Wachstumsplatz")

#   @property
#   def highscore_newcomer(self) -> int:
#       pass

    @property
    def agsx_punkte(self) -> int:
        return self._api_data.get("AGSXPunkte")

    @property
    def in_agsx(self) -> bool:
        return self.highscore_wachstum <= 10 or self.highscore_groesse <= 10

    @property
    def handelsaktivitaet(self) -> int:
        return int(self._api_data.get("Aktivitaet"))

#   @property
#   def ceo(self):
#       pass
#
#   @property
#   def aktien(self):
#       pass
#
#   @property
#   def anleihen(self): #todo This could be possible via the historic bond drawer and historic bond zins
#       pass
#
#   @property
#   def kredite(self):
#       pass
#
#   @property
#   def zertifikate(self):
#       pass
#
#   @property
#   def orders(self):
#       pass
#
#   @property
#   def aktionaere(self):
#       pass
#
#   @property
#   def kes(self):
#       pass
#
#   @property
#   def khs(self):
#       pass
#
#   @property
#   def dividende(self) -> float:
#       pass
#
#   @property
#   def max_zertis(self) -> int:
#       pass
#
#   @property
#   def uebernahmeschutz(self) -> bool:
#       pass
#
#   @property
#   def tages_hoch(self) -> float:
#       pass
#
#   @property
#   def tages_tief(self) -> float:
#       pass
#
#   @property
#   def kurs_14d(self) -> float:
#       pass
#
#   @property
#   def kurs_30d(self) -> float:
#       pass
#
#   @property
#   def kurs_60d(self) -> float:
#       pass
#
#   @property
#   def kurs_90d(self) -> float:
#       pass
#
#   @property
#   def bw_14d(self) -> float:
#       pass
#
#   @property
#   def bw_30d(self) -> float:
#       pass
#
#   @property
#   def bw_60d(self) -> float:
#       pass
#
#   @property
#   def bw_90d(self) -> float:
#       pass
#
#   @property
#   def fp_14d(self) -> float:
#       pass
#
#   @property
#   def fp_30d(self) -> float:
#       pass
#
#   @property
#   def fp_60d(self) -> float:
#       pass
#
#   @property
#   def fp_90d(self) -> float:
#       pass


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

import re
from datetime import datetime, date
from bs4 import BeautifulSoup

from .aktie import Aktie, Aktionaer
from .anleihe import Anleihe, Kredit
from .ceo import Ceo
from .data import Data
from .index import Index
from .kapitalmassnahme import Kapitalerhoehung, Kapitalherabsetzung
from .order import Order
from .zertifikat import Zertifikat


class Ag:
    def __init__(self, wkn: int, api_data: Data, web_data: Data, chronik_data: Data):
        self._wkn: int = wkn
        self._api_data: Data = api_data
        self._web_data: Data = web_data
        self._chronik_data: Data = chronik_data
        self._ag_data = lambda: self._api_data().get("ags").get(str(self.wkn))

    @property
    def wkn(self) -> int:
        return self._wkn

    @property
    def name(self) -> str:
        return self._ag_data().get("name")

    @property
    def logo(self) -> str:
        return "https://www.ag-spiel.de/" + self._web_data().find(id="logocontainer").img.attrs["src"]

    @property
    def gruendung(self) -> datetime:
        return datetime.strptime(self._ag_data().get("gruendung"), "%Y-%m-%d %H:%M:%S")

    @property
    def aktienanzahl(self) -> int:
        return int(self._ag_data().get("aktienanzahl"))

    @property
    def in_liquidation(self) -> bool:
        return self._ag_data().get("in_liquidation")

    @property
    def in_kapitalerhoehung(self) -> bool:
        return "Diese AG befindet sich in einer Kapitalerhöhung" in str(self._web_data())

    @property
    def kurs(self) -> float:
        return float(self._ag_data().get("kurs"))

    @property
    def brief(self) -> float:
        return float(self._ag_data().get("brief"))

    @property
    def geld(self) -> float:
        return float(self._ag_data().get("geld"))

    @property
    def brief_stueckzahl(self) -> int:
        return int(self._ag_data().get("brief_stueckzahl"))

    @property
    def geld_stueckzahl(self) -> int:
        return int(self._ag_data().get("geld_stueckzahl"))

    @property
    def sw_aktie(self) -> float:
        return float(
            re.compile("-?(?:\d{,3}[.])*\d{,3}[,]\d{2}").findall(self._web_data().find("div", attrs={"id": "sw"}).text)[0]
            .replace(".", "").replace(",", "."))

    @property
    def bbw_aktie(self) -> float:
        return float(
            re.compile("-?(?:\d{,3}[.])*\d{,3}[,]\d{2}").findall(self._web_data().find("div", attrs={"id": "bbw"}).text)[0]
            .replace(".", "").replace(",", "."))

    @property
    def fp_aktie(self) -> float:
        return float(
            re.compile("-?(?:\d{,3}[.])*\d{,3}[,]\d{2}").findall(self._web_data().find("div", attrs={"id": "fp"}).text)[0]
            .replace(".", "").replace(",", "."))

    @property
    def bw_aktie(self) -> float:
        return round(self.buchwert / self.aktienanzahl, 2)

    @property
    def kgv(self) -> float:
        return float(re.compile("\d*[,]\d*").findall(self._web_data().find("div", attrs={"id": "kgv"}).text)[0].
                     replace(",", "."))

    @property
    def spread(self) -> float:
        if self.geld == 0 or self.brief == 0:
            return 0
        else:
            return 1 - (self.geld / self.brief)

    @property
    def alter(self) -> int:
        return (datetime.now() - self.gruendung).days

    @property
    def tagesvolumen(self) -> float:
        return Ag._convert_number(self._web_data().find("div", attrs={"id": "tagesvolumen"}).text)

    @property
    def boersenwert(self) -> float:
        return self.kurs * self.aktienanzahl

    @property
    def buchwert(self) -> float:
        return self.depotwert + self.bargeld

    @property
    def depotwert(self) -> float:
        return float(self._ag_data().get("depotwert"))

    @property
    def bargeld(self) -> float:
        return float(self._ag_data().get("bargeld"))

    @property
    def highscore(self) -> int:
        return int(self._ag_data().get("highscore_platz"))

    @property
    def highscore_groesse(self) -> int:
        return int(self._ag_data().get("highscore_platz_groesse"))

    @property
    def highscore_wachstum(self) -> int:
        return int(self._ag_data().get("highscore_platz_wachstum"))

    @property
    def highscore_newcomer(self) -> int:
        return int(self._ag_data().get("highscore_platz_newcomer"))

    @property
    def agsx_punkte(self) -> int:
        return int(self._ag_data().get("agsx_punkte"))

    @property
    def in_agsx(self) -> bool:
        return self._ag_data().get("in_agsx")

    @property
    def handelsaktivitaet(self) -> int:
        return int(self._ag_data().get("handelsaktivitaet"))

    @property
    def ceo(self) -> Ceo:
        name = self._ag_data().get("ceo").get("name")

        # Alles zum Index
        try:
            index_id = int(self._ag_data().get("index_id"))
            index_data = self._api_data().get("indizes").get(str(index_id))
            index_name = index_data.get("name")
            index_highscore = int(index_data.get("highscore_platz"))
            index_punkte = int(index_data.get("punkte"))
            index_gruendung = datetime.strptime(index_data.get("gruendung_datum"), "%Y-%m-%d %H:%M:%S")
            index = Index(nummer=index_id, name=index_name, highscore=index_highscore, punkte=index_punkte,
                          gruendung_datum=index_gruendung)
        except TypeError:  # Wenn Spieler in keinem Index
            index = None

        registrierung_datum = datetime.strptime(self._ag_data().get("ceo").get("registrierung_datum"),
                                                "%Y-%m-%d %H:%M:%S")
        gesperrt = self._ag_data().get("ceo").get("gesperrt")
        userprojekt = self._ag_data().get("ceo").get("ist_userprojekt_account")
        return Ceo(name=name, index=index, registrierung_datum=registrierung_datum, gesperrt=gesperrt,
                   userprojekt=userprojekt)

    @property
    def aktien(self) -> list:
        aktien = []
        for i in self._ag_data().get("aktien"):
            temp = Aktie(wkn=int(i.get("wkn")), stueckzahl=int(i.get("stueckzahl")))
            aktien.append(temp)

        return aktien

    @property
    def anleihen(self) -> list:
        anleihen = []
        for i in self._ag_data().get("anleihen"):
            temp = Anleihe(betrag=int(i.get("betrag")), zins=float(i.get("zins")),
                           auszahlung_datum=datetime.strptime(i.get("auszahlung_datum"), "%Y-%m-%d %H:%M:%S"),
                           laufzeit=int(i.get("laufzeit")))
            anleihen.append(temp)

        return anleihen

    @property
    def kredite(self) -> list:
        kredite = []
        for i in self._ag_data().get("kredite"):
            temp = Kredit(betrag=int(i.get("betrag")), zins=float(i.get("zins")),
                          rueckzahlung_datum=datetime.strptime(i.get("rueckzahlung_datum"), "%Y-%m-%d %H:%M:%S"),
                          laufzeit=int(i.get("laufzeit")))
            kredite.append(temp)

        return kredite

    @property
    def zertifikate(self) -> list:
        zertifikate = []
        for i in self._ag_data().get("zertifikate"):
            try:
                hebel = float(i.get("hebel"))
            except TypeError:  # falls hebel noch nicht bekannt
                hebel = 0
            typ = i.get("typ")
            temp = Zertifikat(betrag=float(i.get("betrag")), typ=typ, hebel=hebel, punkte=int(i.get("punkte")),
                              ablaufdatum=datetime.strptime(i.get("ablauf_datum"), "%Y-%m-%d %H:%M:%S"))
            zertifikate.append(temp)

        return zertifikate

    @property
    def orders(self) -> list:
        orders = []
        for i in self._ag_data().get("orders"):
            temp = Order(typ=i.get("typ"), limit=float(i.get("limit")), stueckzahl=int(i.get("stueckzahl")),
                         orderregel=i.get("orderregel"), systembank=i.get("systembank_order"),
                         datum=datetime.strptime(i.get("datum"), "%Y-%m-%d %H:%M:%S"))
            orders.append(temp)

        return orders

    @property
    def aktionaere(self) -> list:
        aktionaere = []
        for ag in self._api_data().get("ags").values():
            for aktie in ag.get("aktien"):
                if self.wkn in aktie.values():
                    aktionaere.append(Aktionaer(wkn=ag.get("wkn"), stueckzahl=aktie.get("stueckzahl")))

        return aktionaere

    @property
    def kes(self) -> list:
        kes = []
        rows = self._chronik_data().find("table", attrs={"id": "kes"}).find_all("tr")
        for row in rows:
            cols = row.find_all("td")
            if len(cols) > 0:
                if cols[0].text != "Gesamt":
                    datum: date = datetime.strptime(cols[0].text, "%d.%m.%Y").date()
                    anzahl, preis, gesamt = None, None, None
                    if cols[1].text != "n/a":
                        anzahl = int(cols[1].text.replace(".", ""))

                    if cols[2].text != "n/a":
                        preis = float(cols[2].text.replace(".", "").replace(",", ".").replace("€", "").strip())

                    if cols[3].text != "n/a":
                        gesamt = float(cols[3].text.replace(".", "").replace(",", ".").replace("€", "").strip())
                    kes.append(Kapitalerhoehung(datum, anzahl, preis, gesamt))

        return kes

    @property
    def khs(self) -> list:
        khs = []
        rows = self._chronik_data().find("table", attrs={"id": "khs"}).find_all("tr")
        for row in rows:
            cols = row.find_all("td")
            if len(cols) > 0:
                if cols[0].text != "Gesamt":
                    datum: date = datetime.strptime(cols[0].text, "%d.%m.%Y").date()
                    anzahl, preis, gesamt = None, None, None
                    if cols[1].text != "n/a":
                        anzahl = int(cols[1].text.replace(".", ""))

                    if cols[2].text != "n/a":
                        preis = float(cols[2].text.replace(".", "").replace(",", ".").replace("€", "").strip())

                    if cols[3].text != "n/a":
                        gesamt = float(cols[3].text.replace(".", "").replace(",", ".").replace("€", "").strip())
                    khs.append(Kapitalherabsetzung(datum, anzahl, preis, gesamt))

        return khs

    @property
    def dividende(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        return float(re.compile("(0\.?\d{0,2})%").findall(table_data.get("Dividende"))[0])

    @property
    def max_zertis(self) -> int:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        return int(re.compile("(\d{1,2})%").findall(table_data.get("Max. Zertifikatevol."))[0])

    @property
    def uebernahmeschutz(self) -> bool:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        return table_data.get("Hat Übernahmeschutz?").strip() == "ja"

    @property
    def tages_hoch(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        return float(re.compile("\d*\.?\d*,?\d*").findall(table_data.get("Tageshoch"))[0].replace(".", "").
                     replace(",", "."))

    @property
    def tages_tief(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        return float(re.compile("\d*\.?\d*,?\d*").findall(table_data.get("Tagestief"))[0].replace(".", "").
                     replace(",", "."))

    @property
    def kurs_14d(self) -> float:
        return self._get_table_data()[0]

    @property
    def kurs_30d(self) -> float:
        return self._get_table_data()[1]

    @property
    def kurs_60d(self) -> float:
        return self._get_table_data()[2]

    @property
    def kurs_90d(self) -> float:
        return self._get_table_data()[3]

    @property
    def bw_14d(self) -> float:
        return self._get_table_data()[4]

    @property
    def bw_30d(self) -> float:
        return self._get_table_data()[5]

    @property
    def bw_60d(self) -> float:
        return self._get_table_data()[6]

    @property
    def bw_90d(self) -> float:
        return self._get_table_data()[7]

    @property
    def fp_14d(self) -> float:
        return self._get_table_data()[8]

    @property
    def fp_30d(self) -> float:
        return self._get_table_data()[9]

    @property
    def fp_60d(self) -> float:
        return self._get_table_data()[10]

    @property
    def fp_90d(self) -> float:
        return self._get_table_data()[11]

    @property
    def ordner(self) -> str:
        try:
            return re.compile("Ordner:\s(.*)Deine\sNotiz:").findall(
                self._web_data().find("div", attrs={"class": "nodisplay profil-depotcomment"})
                .text)[0]
        except AttributeError:
            return "Allgemein"

    @property
    def kauf_kurs(self) -> float:
        try:
            return float(
                re.compile("Kauf:\s((?:\d*[.,]?)*)\s€").findall(self._web_data().find("div", attrs={"id": "imdepot"})
                                                                .text)[0].replace(".", "").replace(",", "."))
        except AttributeError:
            return None

    @property
    def kauf_anzahl(self) -> int:
        try:
            return int(re.compile("In\sdeinem\sDepot:((?:\d*\.?)*)\sStk\.").findall(
                self._web_data().find("div", attrs={"id": "imdepot"})
                .text)[0].replace(".", ""))
        except AttributeError:
            return 0

    def _get_table_data(self):
        table_data = []
        rows = self._web_data().find('table', attrs={'class': 'normalborder'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            if len(cols) >= 2:
                # Nur hier stehen Werte
                last_col = str(cols[len(cols) - 1])
                regex = re.findall(">-?\d*.?\d*,\d*<", last_col)
                try:
                    table_data.append(float(regex[0][1:-1].replace(".", "").replace(",", ".")))
                except IndexError:
                    table_data.append(None)

        return table_data

    @staticmethod
    def _convert_number(number: str) -> float:
        """
        Diese Methode wandelt Zahlen in der Form "5,6 Mio." in Gleitkommazahlen um.

        :param number: Die umzuwandelnde Zahl
        :return: Die umgewandelte Gleitkommazahl
        """
        ergebnis = float(re.compile("\d+[,]?\d*").findall(number)[0].replace(",", "."))
        multiplier = re.compile("\d*[,]?\d*.(:Mrd\.|Mio\.|Tsd\.)").findall(number)
        if multiplier:
            multiplier = multiplier[0]
            if multiplier == "Mrd.":
                ergebnis *= 1000000000
            elif multiplier == "Mio.":
                ergebnis *= 1000000
            elif multiplier == "Tsd.":
                ergebnis *= 1000
            else:
                raise ValueError()

        return ergebnis


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime


class Index:
    def __init__(self, nummer: int, name: str, highscore: int, punkte: int, gruendung_datum: datetime):
        self._id: int = nummer
        self._name: str = name
        self._highscore: int = highscore
        self._punkte: int = punkte
        self._gruendung_datum: datetime = gruendung_datum

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def highscore(self) -> int:
        return self._highscore

    @property
    def punkte(self) -> int:
        return self._punkte

    @property
    def gruendung_datum(self) -> datetime:
        return self._gruendung_datum


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime, date, timedelta

import json
from typing import Union

from requests import get
from bs4 import BeautifulSoup

from .ag import Ag
from .data import Data
from .markt import Markt
from .historic_ag import HistorischeAg


class AgNotFoundError(Exception): pass


class Api:
    _api_url = "https://www.ag-spiel.de/api/get/data.php?version=5"
    _historic_api_url = "http://api.agscio.de:8080/apiserver/api.rsc"

    def __init__(self, phpsessid: str = "", api_key:str="0t7B1d3c0G5k9i3O8q1j", premium: bool = True):
        self._phpsessid = phpsessid
        self._api_key = api_key
        self.premium = premium
        self._api_data = Data(update=lambda: json.loads(get(Api._api_url).content))

    def get_ag(self, wkn: int, datum:date=None) -> Union[Ag, HistorischeAg]:
        """
        Diese Methode gibt ein Objekt der Klasse Ag mit der übergebenen WKN aus.

        :param wkn: Die WKN der gewünschten Ag
        :param datum: Das Datum von welchem die Ag-Daten stammen sollen
        :return: Ein Objekt der Klasse Ag
        """
        today = date.today()
        if datum is None or datum == today:
            if str(wkn) in self._api_data().get("ags"):
                web_data = Data(
                    update=lambda: BeautifulSoup(
                        get("https://www.ag-spiel.de/index.php?section=profil&aktie={}"
                                     .format(str(wkn)), cookies={"PHPSESSID": self._phpsessid})
                        .content, "html.parser"))
                chronik_data = Data(
                    update=lambda: BeautifulSoup(get("https://www.ag-spiel.de/index.php?section=chronik&wkn={}"
                                                              .format(str(wkn)), cookies={"PHPSESSID": self._phpsessid})
                                                 .content, "html.parser"))
                return Ag(wkn=wkn, api_data=self._api_data, web_data=web_data, chronik_data=chronik_data)
            else:
                raise AgNotFoundError("Die AG mit der WKN " + str(wkn) + " wurde nicht gefunden.")
        elif datum > today:
            raise ValueError("Das Datum liegt in der Zukunft!")
        else:
            link = Api._historic_api_url + f"/Profile_Prem(WKN={wkn}, Datum={datum + timedelta(1)})"
            api_data = get(link, params={"@authtoken": self._api_key}).text
            if api_data == "":
                raise AgNotFoundError("Die AG mit der WKN " + str(wkn) + " wurde zum angegebenen Zeitpunkt nicht gefunden.")
            else:
                return HistorischeAg(wkn, datum, json.loads(api_data))

    def get_all_ags(self) -> list:
        """
        Diese Methode gibt eine Liste mit allen Ag-Objekten aus, die es im AG-Spiel gibt.

        :return: Die Liste aller AGs
        """
        ergebnis = []
        for i in self._api_data().get("ags"):
            ergebnis.append(self.get_ag(int(i)))

        return ergebnis

    def get_markt(self) -> Markt:
        web_data = Data(
            update=lambda: BeautifulSoup(get("https://www.ag-spiel.de/index.php?section=statistiken",
                                                      cookies={"PHPSESSID": self._phpsessid}).content, "html.parser"))
        return Markt(api_data=self._api_data, web_data=web_data)

    @property
    def api_version(self) -> int:
        return int(self._api_data().get("api_version"))

    @property
    def daten_datum(self) -> datetime:
        return datetime.strptime(self._api_data().get("daten_datum"), "%Y-%m-%d %H:%M:%S")


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from agspiel.api.api import Api
from datetime import datetime
from unittest import TestCase

class TestApi(TestCase):
    def setUp(self):
        self.api = Api("")

    def test_api_version(self):
        self.assertIsInstance(self.api.api_version, int)

    def test_daten_datum(self):
        self.assertIsInstance(self.api.daten_datum, datetime)


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import date


class _Kapitalmassnahme:
    def __init__(self, datum: date, stueckzahl: int, kurs: float, summe: float):
        self._datum: date = datum
        self._stueckzahl: int = stueckzahl
        self._kurs: float = kurs
        self._summe: float = summe

    @property
    def datum(self) -> date:
        return self._datum

    @datum.setter
    def datum(self, datum: date):
        self._datum = datum

    @property
    def stueckzahl(self) -> int:
        return self._stueckzahl

    @stueckzahl.setter
    def stueckzahl(self, stueckzahl: int):
        self._stueckzahl = stueckzahl

    @property
    def kurs(self) -> float:
        return self._kurs

    @kurs.setter
    def kurs(self, kurs: float):
        self._kurs = kurs

    @property
    def summe(self) -> float:
        return self._summe

    @summe.setter
    def summe(self, summe: float):
        self._summe = summe


class Kapitalerhoehung(_Kapitalmassnahme):
    pass


class Kapitalherabsetzung(_Kapitalmassnahme):
    pass


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

import re

from .data import Data


class Markt:
    def __init__(self, api_data: Data, web_data: Data):
        self._api_data = api_data
        self._web_data = web_data
        self._markt_data = lambda: self._api_data().get("allgemein")

    @property
    def ags(self) -> int:
        return int(self._markt_data().get("ags"))

    @property
    def orders_24(self) -> int:
        return int(self._markt_data().get("24_stunden_orders"))

    @property
    def volumen_24(self) -> float:
        return float(self._markt_data().get("24_stunden_ordervolumen"))

    @property
    def agsx_punkte(self) -> int:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        return int(table_data.get("Punktestand").replace(".", ""))

    @property
    def agsx_aenderung(self) -> int:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        return int(table_data.get("Änderung"))

    @property
    def put_hebel(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        return float(re.compile("(\d\.?\d*)\s/\s\d\.?\d*").findall(table_data.get("Put / Call"))[0])

    @property
    def call_hebel(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        return float(re.compile("\d\.?\d*\s/\s(\d\.?\d*)").findall(table_data.get("Put / Call"))[0])

    @property
    def anleihenzins(self) -> float:
        table_data = {}
        rows = self._web_data().find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        return float(re.compile("(\d\.?\d*)%").findall(table_data.get("Anleihezins"))[0])


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime

from .index import Index


class Ceo:
    def __init__(self, name: str, index: Index, registrierung_datum: datetime, gesperrt: bool, userprojekt: bool):
        self._name: str = name
        self._index: Index = index
        self._registrierung_datum: datetime = registrierung_datum
        self._gesperrt: bool = gesperrt
        self._userprojekt: bool = userprojekt

    @property
    def name(self) -> str:
        return self._name

    @property
    def index(self) -> Index:
        return self._index

    @property
    def registrierung_datum(self) -> datetime:
        return self._registrierung_datum

    @property
    def gesperrt(self) -> bool:
        return self._gesperrt

    @property
    def userprojekt(self) -> bool:
        return self._userprojekt


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from requests import Session

class LoginError(Exception):
    pass

def get_php_session_id(email:str, password:str) -> str:
    """
    Diese Funktion loggt sich im AG-Spiel ein, liest den Cookie "PHPSESSID" aus
    und gibt diesen an den Aufrufer zurück. Für den Fall das die Logindaten
    fehlerhaft sind, wird ein LoginError erzeugt.

    :param email: E-Mail Adresse des AGS-Accounts
    :param password: Passwort des AGS-Accounts
    :raises LoginError: Anmeldedaten sind falsch
    :return: Wert des PHPSESSID Cookies
    """
    request_data = {"email": email, "userpass": password, "permanent": "1", "login": "Einloggen"}
    response = Session().post("https://www.ag-spiel.de/index.php?section=login", data=request_data)
    if "Es gibt keinen Benutzer mit dieser E-Mail!" in response.text:
        raise LoginError("Keinen Benutzer mit dieser Mail!")
    elif "Das Passwort ist nicht korrekt!" in response.text:
        raise LoginError("Falsches Passwort!")
    else:
        return response.cookies.get_dict()["PHPSESSID"]


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

import json
from unittest import TestCase
from bs4 import BeautifulSoup
from agspiel.api.markt import Markt
from agspiel.api.data import Data

class TestMarkt(TestCase):
    def setUp(self):
        f = open("testmarkt.txt", "rb")
        web_data = Data(data=BeautifulSoup(f.read(), "html.parser"), update=lambda: self.data)
        f.close()
        f = open("testapi.txt", "r")
        api_data = Data(data=json.loads(f.read()), update=lambda: self.data)
        f.close()
        self.markt = Markt(api_data=api_data, web_data=web_data)

    def test_ags(self):
        self.assertEqual(self.markt.ags, 750)
        self.assertIsInstance(self.markt.ags, int)

    def test_orders_24(self):
        self.assertEqual(self.markt.orders_24, 2598)
        self.assertIsInstance(self.markt.orders_24, int)

    def test_volumen_24(self):
        self.assertEqual(self.markt.volumen_24, 957984387.62)
        self.assertIsInstance(self.markt.volumen_24, float)

    def test_agsx_punkte(self):
        self.assertEqual(self.markt.agsx_punkte, 23332)
        self.assertIsInstance(self.markt.agsx_punkte, int)

    def test_agsx_aenderung(self):
        self.assertEqual(self.markt.agsx_aenderung, -42)
        self.assertIsInstance(self.markt.agsx_aenderung, int)

    def test_put_hebel(self):
        self.assertEqual(self.markt.put_hebel, 4.3)
        self.assertIsInstance(self.markt.put_hebel, float)

    def test_call_hebel(self):
        self.assertEqual(self.markt.call_hebel, 1.04)
        self.assertIsInstance(self.markt.call_hebel, float)

    def test_anleihenzins(self):
        self.assertEqual(self.markt.anleihenzins, 1.5)
        self.assertIsInstance(self.markt.anleihenzins, float)


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

class Aktie:
    def __init__(self, wkn: int, stueckzahl: int):
        self.wkn: int = wkn
        self.stueckzahl: int = stueckzahl

    @property
    def wkn(self) -> int:
        return self._wkn

    @wkn.setter
    def wkn(self, value: int):
        self._wkn = value

    @property
    def stueckzahl(self) -> int:
        return self._stueckzahl

    @stueckzahl.setter
    def stueckzahl(self, value: int):
        self._stueckzahl = value


class Aktionaer(Aktie):
    def __init__(self, wkn: int, stueckzahl: int):
        super().__init__(wkn, stueckzahl)


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from decimal import Decimal
from requests import post, get
from datetime import datetime
from re import search

from bs4 import BeautifulSoup

from agspiel.utils.errors import OrderCreationException, OrderDeletionException, OrderException

class Controller:
    def __init__(self, phpsessid:str):
        self._phpsessid = phpsessid
        soup = BeautifulSoup(get("https://www.ag-spiel.de/index.php?section=agorderbuch",
                                 cookies={"PHPSESSID": self._phpsessid}).content, "html.parser")
        self._token = soup.find("input", attrs={"type":"hidden", "name":"token"}).get("value")

    def create_order(self, wkn:int, buy:bool, limit:float, anzahl:int, tage_gueltig:float=14.00, gueltig_ab:datetime=datetime.now(), **kwargs) -> int:
        orderwert = str(Decimal(str(limit)) * anzahl)
        body = {
            "privat": 0,
            "aktie": wkn,
            "limit": limit,
            "anzahl": anzahl,
            "orderwert": orderwert,
            "gab_tag": gueltig_ab.day,
            "gab_monat": gueltig_ab.month,
            "gab_jahr": gueltig_ab.year,
            "gab_stunde": gueltig_ab.hour,
            "gab_minute": gueltig_ab.minute,
            "gueltigkeit": tage_gueltig,
            "token": self._token
        }

        if buy:
            body["order"] = "buy"
        else:
            body["order"] = "sell"

        if "aenderung" in kwargs:
            body["aenderung"] = kwargs.get("aenderung")
            body["aendern"] = 1
            if "stop" in kwargs:
                body["aenderung_stop"] = kwargs.get("stop")
                body["aendern2"] = 1
            else:
                body["aenderung_stop"] = limit
        else:
            body["aenderung"] = 0.0005

        response = post("https://www.ag-spiel.de/index.php?section=agorderbuch&action=create&ele=", body, cookies={"PHPSESSID": self._phpsessid})
        soup = BeautifulSoup(response.content, "html.parser")
        result = self._get_infobox_text(soup)

        if result != "Ordererstellung beauftragt.":
            raise OrderCreationException(result)

        cross = soup.find("a", attrs={"class": "button cross notext", "title": "Order löschen/bearbeiten"},
                          href=lambda x: f"&action=delete&aktie={wkn}&limit={'%.2f' % limit}&anzahl={anzahl}" in x)
        orderid = search("index\.php\?section=agorderbuch&orderid=(\d*)&", cross.get("href"))
        if not orderid:
            raise OrderException("Order wurde erstellt, jedoch wurde die OrderID nicht gefunden.")

        return int(orderid.group(1))


    def delete_order(self, orderid:int) -> bool:
        # Get link by orderid is not needed cause we only need the orderid
        # soup = BeautifulSoup(get("https://www.ag-spiel.de/index.php?section=agorderbuch",
        #                          cookies={"PHPSESSID": self._phpsessid}).content, "html.parser")
        # cross = soup.find("a", attrs={"class":"button cross notext", "title":"Order löschen/bearbeiten"}, href=lambda x: f"index.php?section=agorderbuch&orderid={orderid}&action=delete&aktie=" in x)
        # link = "https://www.ag-spiel.de/" + cross.get("href")
        response = get(f"https://www.ag-spiel.de/index.php?section=agorderbuch&orderid={orderid}&action=delete", cookies={"PHPSESSID": self._phpsessid})
        result = self._get_infobox_text(BeautifulSoup(response.content, "html.parser"))
        if result != "Order wurde gelöscht.":
            raise OrderDeletionException(result)

        return True

    @classmethod
    def _get_infobox_text(cls, soup:BeautifulSoup) -> str:
        infobox = soup.find("div", attrs={"class": "infobox"})
        result = str()
        for child in infobox.children:
            if child.name is None:
                if result != "":
                    result += " "

                result += child.strip()

        return result


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime


class Order:
    def __init__(self, typ: str, limit: float, stueckzahl: int, orderregel: bool, systembank: bool, datum: datetime):
        self.typ: str = typ
        self.limit: float = limit
        self.stueckzahl: int = stueckzahl
        self.orderregel: bool = orderregel
        self.systembank: bool = systembank
        self.datum: datetime = datum

    @property
    def typ(self) -> str:
        return self._typ

    @typ.setter
    def typ(self, value: str):
        self._typ = value

    @property
    def limit(self) -> float:
        return self._limit

    @limit.setter
    def limit(self, value: float):
        self._limit = value

    @property
    def stueckzahl(self) -> int:
        return self._stueckzahl

    @stueckzahl.setter
    def stueckzahl(self, value: int):
        self._stueckzahl = value

    @property
    def orderregel(self) -> bool:
        return self._orderregel

    @orderregel.setter
    def orderregel(self, value: bool):
        self._orderregel = value

    @property
    def systembank(self) -> bool:
        return self._systembank

    @systembank.setter
    def systembank(self, value: bool):
        self._systembank = value

    @property
    def datum(self) -> datetime:
        return self._datum

    @datum.setter
    def datum(self, value: datetime):
        self._datum = value


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime


class Zertifikat:
    def __init__(self, betrag: float, typ: str, hebel: float, punkte: int, ablaufdatum: datetime):
        self.betrag: float = betrag
        self.typ: str = typ
        self.hebel: float = hebel
        self.punkte: int = punkte
        self.ablaufdatum: datetime = ablaufdatum

    @property
    def betrag(self) -> float:
        return self._betrag

    @betrag.setter
    def betrag(self, value: float):
        self._betrag = value

    @property
    def typ(self) -> str:
        return self._typ

    @typ.setter
    def typ(self, value: str):
        self._typ = value

    @property
    def hebel(self) -> float:
        return self._hebel

    @hebel.setter
    def hebel(self, value: float):
        self._hebel = value

    @property
    def punkte(self) -> int:
        return self._punkte

    @punkte.setter
    def punkte(self, value: int):
        self._punkte = value

    @property
    def ablaufdatum(self) -> datetime:
        return self._ablaufdatum

    @ablaufdatum.setter
    def ablaufdatum(self, value: datetime):
        self._ablaufdatum = value


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime


class Data:
    def __init__(self, update, data=None):
        self.data = data
        self.update = update  # type(update) -> function
        self.timestamp = datetime.now()

    def _update_data(self):
        self.data = self.update()

    @property
    def data(self):
        if self._data is None:
            self._update_data()
        else:
            now = datetime.now()
            now = (now.minute + (now.hour * 60) + (now.day * 24 * 60)) % 5
            compare = (self.timestamp.minute + (self.timestamp.hour * 60) + (self.timestamp.day * 24 * 60)) % 5
            if now > compare:
                self._update_data()

        return self._data

    @data.setter
    def data(self, value):
        self._data = value
        self.timestamp = datetime.now()

    @property
    def update(self):
        return self._update

    @update.setter
    def update(self, value):
        self._update = value

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value: datetime):
        self._timestamp = value

    def __call__(self, *args, **kwargs):
        return self.data


#  Copyright (c) 2021 | KingKevin23 (@kingkevin023)

from datetime import datetime


class _Superclass:
    def __init__(self, betrag: int, zins: float, laufzeit: int):
        self.betrag: int = betrag
        self.zins: float = zins
        self.laufzeit: int = laufzeit

    @property
    def betrag(self) -> int:
        return self._betrag

    @betrag.setter
    def betrag(self, value: int):
        self._betrag = value

    @property
    def zins(self) -> float:
        return self._zins

    @zins.setter
    def zins(self, value: float):
        self._zins = value

    @property
    def laufzeit(self) -> int:
        return self._laufzeit

    @laufzeit.setter
    def laufzeit(self, value: int):
        self._laufzeit = value


class Anleihe(_Superclass):
    def __init__(self, betrag: int, zins: float, auszahlung_datum: datetime, laufzeit: int):
        super().__init__(betrag, zins, laufzeit)
        self.auszahlung_datum: datetime = auszahlung_datum

    @property
    def auszahlung_datum(self) -> datetime:
        return self._auszahlung_datum

    @auszahlung_datum.setter
    def auszahlung_datum(self, value: datetime):
        self._auszahlung_datum = value


class Kredit(_Superclass):
    def __init__(self, betrag: int, zins: float, rueckzahlung_datum: datetime, laufzeit: int):
        super().__init__(betrag, zins, laufzeit)
        self.rueckzahlung_datum: datetime = rueckzahlung_datum

    @property
    def rueckzahlung_datum(self) -> datetime:
        return self._rueckzahlung_datum

    @rueckzahlung_datum.setter
    def rueckzahlung_datum(self, value: datetime):
        self._rueckzahlung_datum = value